@@ -23,6 +23,8 @@
 #include "acer.h"
 #include "protocols/necprotocol.h"
 
+#include <QComboBox>
+
 AcerTV1::AcerTV1(
   unsigned int index)
   : PIRKeysetMetaData(
@@ -105,6 +107,16 @@ void AcerTV1::populateProtocol(
 }
 
 
+void AcerTV1::populateInputList(
+  QComboBox *cb)
+{
+  cb->clear();
+
+  cb->addItem("SCART", QVariant(ScartInput_Key));
+  cb->addItem("PC", QVariant(PCInput_Key));
+}
+
+
 AcerPC1::AcerPC1(
   unsigned int index)
   : PIRKeysetMetaData(
