@@ -22,6 +22,7 @@
 
 #include "bush.h"
 #include "protocols/rc5protocol.h"
+#include <QComboBox>
 
 
 BushTV1::BushTV1(
@@ -80,6 +81,16 @@ void BushTV1::populateProtocol(
 }
 
 
+void BushTV1::populateInputList(
+  QComboBox *cb)
+{
+  cb->clear();
+
+  cb->addItem("TV", QVariant(Input_Key));
+  cb->addItem("Aux", QVariant(AuxInput_Key));
+}
+
+
 BushTV2::BushTV2(
   unsigned int index)
   : PIRKeysetMetaData(
