@@ -23,6 +23,8 @@
 #include "sharp.h"
 #include "protocols/sharpprotocol.h"
 #include "protocols/necprotocol.h"
+#include <QComboBox>
+
 
 SharpTV1::SharpTV1(
   unsigned int index)
@@ -452,6 +454,18 @@ void SharpReceiver1::populateProtocol(
 }
 
 
+void SharpReceiver1::populateInputList(
+  QComboBox *cb)
+{
+  cb->clear();
+
+  cb->addItem("Tuner", QVariant(TunerInput_Key));
+  cb->addItem("CD", QVariant(CDInput_Key));
+  cb->addItem("Tape", QVariant(TapeInput_Key));
+  cb->addItem("Phono", QVariant(PhonoInput_Key));
+}
+
+
 SharpAC1::SharpAC1(
   unsigned int index)
   : PIRKeysetMetaData(
